package cytoscape.dialogs;

import cytoscape.logger.CyLogger;

public class ErrorDialog extends javax.swing.JDialog {
    
	private static final String defaultTitle = "Cytoscape Errors";
	private java.util.List<String> errors;
	
    /** Creates new form ErrorDialog */
    public ErrorDialog() {
        super();
        setTitle(defaultTitle);
        setModal(false);
    	initComponents();
    	errors = new java.util.ArrayList<String>();
    }

    public ErrorDialog(javax.swing.JDialog parent) {
    	super(parent, defaultTitle, false);
    	initComponents();
    	errors = new java.util.ArrayList<String>();
    }

    public ErrorDialog(javax.swing.JDialog parent, String title) {
        super(parent, title, false);
    	initComponents();
    	errors = new java.util.ArrayList<String>();
    }
    
    public ErrorDialog(javax.swing.JFrame parent, String title) {
    	super(parent, title, false);
    	initComponents();
    	errors = new java.util.ArrayList<String>();
    }

    public ErrorDialog(javax.swing.JFrame parent) {
    	super(parent, defaultTitle, false);
    	initComponents();
    	errors = new java.util.ArrayList<String>();
    }
    
    
    public void addError(String e) {
    	errors.add(e);
    	renderErrors();
    }
    
    public void addError(Throwable t) {
    	errors.add(t.toString());
    	renderErrors();
    }

    public boolean hasErrors() {
    	return (errors.size() > 0)? true: false; 
    	
    }
    
    private void renderErrors() {
    	errorPane.setContentType("text/html");
		
    	StringBuffer sb = new StringBuffer();
		sb.append("<html><style type='text/css'>");
		sb.append("body,th,td,div,p,h1,h2,li,dt,dd ");
		sb.append("{ font-family: Tahoma, \"Gill Sans\", Arial, sans-serif; }");
		sb.append("body { margin: 0px; color: #333333; background-color: #ffffff; }");
		sb.append("#indent { padding-left: 30px; }");
		sb.append("ul {list-style-type: none}");
		sb.append("</style><body>");
 
		sb.append("<table width='100%' cellspacing='5'>");
		
		int line = 1;
    	for (int i=errors.size()-1; i >= 0; i--) {
    		sb.append("<tr><td width='5%'>" + line + "</td><td width='95%'>");
    		sb.append(errors.get(i));
    		sb.append("</td></tr><tr><td colspan='2'><hr></td></tr>");
    		line++;
    	}
    	sb.append("</table></body></html>");
    	errorPane.setText(sb.toString());
    }
    
	private javax.swing.ImageIcon createImageIcon(String path,
			String description) {
		java.net.URL imgURL = getClass().getResource(path);
		if (imgURL != null) {
			return new javax.swing.ImageIcon(imgURL, description);
		} else {
			CyLogger.getLogger().warn("Couldn't find file: " + path);
			return null;
		}
	}

    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">                          
    private void initComponents() {
        closeButton = new javax.swing.JButton();
        jScrollPane1 = new javax.swing.JScrollPane();
        errorPane = new javax.swing.JEditorPane();
        errorLabel = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        closeButton.setText("Close");
        closeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                closeButtonActionPerformed(evt);
            }
        });

        errorPane.setEditable(false);
        jScrollPane1.setViewportView(errorPane);

        errorLabel.setIcon( createImageIcon("/cytoscape/images/ximian/stock_dialog-warning-32.png", "Warning"));
        errorLabel.setText(" Warning!");

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 368, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, closeButton)
                    .add(errorLabel))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(errorLabel)
                .add(8, 8, 8)
                .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 241, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(closeButton)
                .addContainerGap())
        );
        pack();
    }// </editor-fold>                        


    private void closeButtonActionPerformed(java.awt.event.ActionEvent evt) {                                            
    	this.dispose();
    }                                           
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
    	ErrorDialog ed = new ErrorDialog();
    	
    	ed.addError("Foo bar");
    	ed.addError("Well damn");
    	
    	ed.setVisible(true);
    }
    
    // Variables declaration - do not modify                     
    private javax.swing.JButton closeButton;
    private javax.swing.JEditorPane errorPane;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JLabel errorLabel;
    // End of variables declaration                   
    
}
